import uuid

from datetime import datetime
from typing import TYPE_CHECKING, List

from sqlalchemy import DateTime, ForeignKey, Integer, text
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .food_table import FoodTable
    from .inventory import Inventory
    from .user import User
    from .voucher import Voucher


@reg.mapped_as_dataclass(unsafe_hash=True)
class Kot:
    __tablename__ = "kots"

    id: Mapped[uuid.UUID] = mapped_column(
        "id", UUID(as_uuid=True), primary_key=True, server_default=text("gen_random_uuid()"), insert_default=uuid.uuid4
    )
    voucher_id: Mapped[uuid.UUID] = mapped_column(
        "voucher_id",
        UUID(as_uuid=True),
        ForeignKey("vouchers.id"),
        nullable=False,
        index=True,
    )
    code: Mapped[int] = mapped_column("code", Integer, nullable=False, unique=True)
    food_table_id: Mapped[uuid.UUID] = mapped_column(
        "food_table_id",
        UUID(as_uuid=True),
        ForeignKey("food_tables.id"),
        nullable=False,
    )
    date: Mapped[datetime] = mapped_column("date", DateTime, nullable=False, index=True)
    user_id: Mapped[uuid.UUID] = mapped_column("user_id", UUID(as_uuid=True), ForeignKey("users.id"), nullable=False)

    voucher: Mapped["Voucher"] = relationship(back_populates="kots")
    user: Mapped["User"] = relationship(backref="kots")
    food_table: Mapped["FoodTable"] = relationship(backref="kots")
    inventories: Mapped[List["Inventory"]] = relationship(back_populates="kot")

    def __init__(
        self,
        voucher_id=None,
        code=None,
        food_table_id=None,
        date=None,
        user_id=None,
        id_=None,
    ):
        self.id = id_
        self.voucher_id = voucher_id
        self.code = code
        self.food_table_id = food_table_id
        self.date = date
        self.user_id = user_id
